import logging

class ImageSearch:
    __logger = logging.getLogger(__name__)

    def __init__(self, feature_extractor, milvus):
        self.feature_extractor = feature_extractor
        self.milvus = milvus

    def image_to_image_search(self, bucket, image, top_k = 100):
        try:
            # 提取查询图像的特征
            vector = self.feature_extractor.extract_from_url(image)

            results = self.milvus.search(bucket, vector, top_k)

            # 处理结果
            if not results or len(results) == 0:
                return []

            # 返回结果
            keys = [hit.entity.get("key") for hit in results[0]]
            images = [hit.entity.get("image") for hit in results[0]]
            scores = [hit.score for hit in results[0]]

            return list(zip(keys, images, scores))

        except Exception as e:
            self.__logger.error(f"Image search failed: {e}")
            raise ValueError(f"Image search failed: {e}")