from typing import Dict, List, Any, Optional, Union
import numpy as np
from pymilvus import connections, Collection, FieldSchema, CollectionSchema, DataType, utility

# , collection_name: str = "image_collection"
class MilvusClient:
    def __init__(self, host = "localhost", port = "19530"):
        self.host: str = host
        self.port: str = port

    def connect(self, alias = "default") -> 'MilvusClient':
        connections.connect(alias, host=self.host, port=self.port)
        return self

    # # 定义集合结构
    # fields: List[FieldSchema] = [
    #     FieldSchema(name="image", dtype=DataType.VARCHAR, max_length=256, is_primary=True, auto_id=False),
    #     FieldSchema(name="product_id", dtype=DataType.VARCHAR, max_length=256),
    #     FieldSchema(name="embedding", dtype=DataType.FLOAT_VECTOR, dim=512)  # CLIP ViT-B/32的特征维度为512
    # ]
    @staticmethod
    def create_collection(collection_name, fields, description):
        schema: CollectionSchema = CollectionSchema(fields, description)
        # 创建集合
        Collection(name=collection_name, schema=schema)

    @staticmethod
    def get_collection(collection_name):
        if not utility.has_collection(collection_name):
            raise RuntimeError(f"集合 '{collection_name}' 不存在")
        return Collection(name=collection_name)

    # index_params: Dict[str, Any] = {
    #     "index_type": "HNSW",
    #     "metric_type": "IP",  # 内积相似度
    #     "params": {"M": 16, "efConstruction": 200}
    # }
    def create_index(self, collection_name, field_name, index_params) -> None:
        self.get_collection(collection_name).create_index(field_name=field_name, index_params=index_params)

    # anns_field = embedding
    # output_fields=["image", "product_id"]
    # search_params: Dict[str, Any] = {"metric_type": "IP", "params": {"ef": 100}}

    def search(self, collection_name, vector, top_k = 10) -> Any:
        collection = self.get_collection(collection_name)

        search_params: Dict[str, Any] = {"metric_type": "IP", "params": {"ef": 100}}

        results = collection.search(
            data=[vector],
            anns_field="vector",
            param=search_params,
            limit=top_k,
            output_fields=["image", "key"]
        )
        return results

    # entities: List[List[Any]] = [
    #     [item["image"] for item in data],
    #     [item["product_id"] for item in data],
    #     [item["embedding"] for item in data]
    # ]
    def insert(self, collection_name, entities):
        self.get_collection(collection_name).insert(entities)

    @staticmethod
    def close(alias ="default") -> None:
        connections.disconnect(alias)
